#include "globals.h"


void uartRxHandler(unsigned char c)
{
	lcd_home();
	lcd_string2("PC char:%c   ",c);
	char xs[255] = {0};
	sprintf(xs, "uC Echo:%c [%d]\n\r",c,c);
	uartSendBuffer(xs, strlen(xs));
}



int main(void)
{
	lcd_init();								// init the LCD screen
	lcd_clrscr();							// initial screen cleanup
	lcd_string2("Init, please wait");

	PUSHBUTTONInit();						// init the pushbutton pins

	a2dInit(); 								// initialize analog to digital converter (ADC)
	a2dSetPrescaler(ADC_PRESCALE_DIV32); 	// configure ADC scaling
	a2dSetReference(ADC_REFERENCE_AVCC); 	// configure ADC reference voltage

	//uart for debugging
	uartInit();  							// initialize the UART (serial port)
	uartSetBaudRate(9600);					// set the baud rate of the UART for our debug/reporting output
	uartSetRxHandler(uartRxHandler);		// set the receive proc
	uartSendBuffer("Serial on.\n\r",12);	

	char screenbuffer[2][16];				// screen buffer
	int i=0;

	//game vars
	int shipindex = 0;
	int enemy[2][16] = {0};

	//game loop
	while(1)
	{
		//LCD screen
		lcd_home();

		//read user data
		int pot = a2dConvert8bit(POT_DQ) - 22; //PC5: 0_232
		int but1 = PUSHBUTTON1Status();
		int but2 = PUSHBUTTON2Status();
				
		//setup screen buffer
		memset(screenbuffer,' ',sizeof(screenbuffer));
		if (but1) shipindex = 0;										//move user ship
		if (but2) shipindex = 1;

		//compute enemy
		if (enemy[0][0]) 
		{
			if (shipindex == 0) 
			{ 
				lcd_clrscr();
				lcd_string2("OUCH-GAME OVER\n  press RESET  "); 
				return 1;  
			}
			enemy[0][0] = 0;											//remove left position enemy
		}
		if (enemy[1][0])
		{
			if (shipindex == 1) 
			{ 
				lcd_clrscr();
				lcd_string2("OUCH-GAME OVER\n  press RESET  "); 
				return 1;  
			}
			enemy[1][0] = 0;											//remove left position enemy
		}
		for (int j=0;j<15;j++)
		{
			if (enemy[0][j+1]) { enemy[0][j] = 1; enemy[0][j+1] = 0; }	//advance row1
			if (enemy[1][j+1]) { enemy[1][j] = 1; enemy[1][j+1] = 0; }	//advance row2
		}
		int rand1 = rand() % 10;
		int rand2 = rand() % 10;
		if (rand1 == 5) enemy[0][15] = 1;								//add new enemy row1
		else
			if (rand2 == 5) enemy[1][15] = 1;							//add new enemy row2
		for(int j=0;j<15;j++)											//add enemy to screen buffer
		{
			if (enemy[0][j]) screenbuffer[0][j] = '@';
			if (enemy[1][j]) screenbuffer[1][j] = '@';
		}
		//draw ship
		screenbuffer[shipindex][0] = '>';
		
		//draw score
		if (i<10) screenbuffer[0][15] = '0'+i;
		else if (i<100) 
		{
			screenbuffer[0][14] = '0'+(i / 10);
			screenbuffer[0][15] = '0'+(i % 10);
		}
		//print screen buffer
		lcd_string2("%s\n%s", screenbuffer[0], screenbuffer[1]);
		//lcd_string2("%d %d", rand1,rand2);

		//some delay
		delay_us(300000);

		i++;
		if (i==99) 
		{
			lcd_clrscr();
			lcd_string2("    YOU WON!   \n  press RESET  "); 
			return 1;  
		}
	}

	return 1;
}
